import java.awt.*;
import java.applet.*;
import java.io.*;

/******************************************************************************/
/* class Card                                                                 */
/*                                                                            */
/* Represents a card.                                                         */
/******************************************************************************/
public class Card 
{
  public static final int HEIGHT = 97;
  public static final int WIDTH = 73;
  
  Image front;  // its picture
  Suit  suit;
  int   value;
    
  /****************************************************************************/
  /* Constructor - load an image                                              */
  /****************************************************************************/
  public Card(Applet applet, Suit suit, int value) 
  {
    this.suit = suit;
    this.value = value;
    
    // Load the image
    try 
    {
      front = applet.getImage(applet.getDocumentBase(), 
                              getCardFilename(suit, value));
    }
    catch (Exception e) 
    {
      System.err.println("Error getting card image: " + e.getMessage());
    }
  }

  /****************************************************************************/
  /* getter methods.                                                          */
  /****************************************************************************/
  int getValue() { return value; }
  Suit getSuit() { return suit; }
  
  /****************************************************************************/
  /* Simple boolean colour matching                                           */
  /****************************************************************************/
  boolean getColour()
  {
    return (suit == Suit.Clubs || suit == Suit.Spades);
  }
        
  /****************************************************************************/
  /* Draw this card at the given coordinates.                                 */
  /****************************************************************************/
  public void draw(Graphics g, int x, int y, Component c) 
  {
    g.drawImage(front, x, y, c);
  }

  /****************************************************************************/
  /* Helper function to calculate filename of image file for a card           */
  /****************************************************************************/
  protected String getCardFilename(Suit s, int value) 
  {
    // the directory where the card images are to be found
    String directory = new String("cards");

    if (value < 10)
    {
      return directory + "/0" + value + s.getLetter() + ".gif";
    }
    else
    {
      return directory + "/"  + value + s.getLetter() + ".gif";
    }
  }
}

