import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.Vector;
import java.util.Enumeration;

/******************************************************************************/
/* class Column                                                               */
/*                                                                            */
/* Represents one of the seven columns of cards in the lower part of the      */
/* playing area.                                                              */
/******************************************************************************/
public class Column extends Pile
{
  // Number of cards which are face down
  int hidden = 0;
  
  /****************************************************************************/
  /* Constructor - make only the first card visible.                          */
  /****************************************************************************/
  Column(Vector v)
  {    
    this.cards = v;
    hidden = count() - 1;
  }
  
  /****************************************************************************/
  /* What we do in response to a mousedown - if they've clicked on a          */
  /* floatable card, return the floated stack.                                */
  /****************************************************************************/
  Vector getFloated(int x, int y, Point offset)
  {
    // A click turns over the bottom card if there are no visible cards.
    // This is a little bit of a hack.
    if (hidden == count() && tableau.getFloater().isEmpty())
    {
      hidden--;
      return null;
    }
    
    // Check for clicks below the cards
    if (y > ((count() - 1) * Y_OFFSET) + Card.HEIGHT)
    {
      return null;
    }

    int clicked_card = getClickedCard(y);
        
    // Clicking a hidden card does not create a floater.
    if (clicked_card < hidden)
    {
      return null;
    }
    
    Vector retval = splitFrom(clicked_card);
    offset.y = offset.y - (clicked_card * Y_OFFSET);
    
    return retval;      
  }
  
  /****************************************************************************/
  /* Split off a certain number of cards and return them (used for floating.) */
  /****************************************************************************/
  protected Vector splitFrom(int split_card)
  {
    // Assumes split_card is a card in this vector;
    Vector retval = new Vector();
    int i;
    int size = count();
    
    // Add the split-off cards to the returnable Vector...
    for (i = split_card; i < size; i++)
    {
      retval.addElement(cards.elementAt(i));
    }
    
    // ...and remove them from the pile.
    while (count() > split_card)
    {
      cards.removeElementAt(count() - 1);
    }
    
    return retval;
  }
  
  /****************************************************************************/
  /* Work out which card was clicked on, given a y-coordinate.                */
  /****************************************************************************/
  protected int getClickedCard(int y)
  {
    // Card clicked, as index into cards Vector.
    int retval = (int)Math.floor((y / Y_OFFSET));
    
    // Fix the value being too great if we're on the bottom card.
    if (retval > count() - 1)       
    {
      retval = count() - 1;
    }

    return retval;     
  }  
  
  /****************************************************************************/
  /* Tell the system what size we want to be.                                 */
  /****************************************************************************/
  public Dimension getPreferredSize()
  {
    return new Dimension(Card.WIDTH, (18 * Y_OFFSET) + Card.HEIGHT);
  }
  
  /****************************************************************************/
  /* Are we allowed to accept the current float, or does it have to go back?  */
  /****************************************************************************/
  boolean floatIsLegal()
  {    
    Card b = tableau.getFloater().bottom();
    
    // If we've got no cards, and it's a King, that's OK
    if (isEmpty())
    {
      if (b.getValue() == 13)
      {      
        return true;
      }
      else
      {
        return false;
      }
    }
    
    // Otherwise, we need a colour mismatch and the number to be one less
    Card t = top();
    
    if ((b.getColour() != t.getColour()) &&
        (b.getValue() == t.getValue() - 1))
    {
      return true;
    }
    
    // None of the OK scenarios apply
    return false;
  }
  
  /****************************************************************************/
  /* paint - draw all the cards in a vertical column.                         */
  /****************************************************************************/
  public void paint(Graphics g)
  {
    int y = 0;
    int hid = 0;
    
    Enumeration e = cards.elements();
    
    while (e.hasMoreElements())
    {
      if (hid < hidden)
      {
        // Card is hidden - draw the back
        g.drawImage(back, 0, y, this);
        e.nextElement();
      }
      else
      {      
        // Card is visible - ask it to draw itself.
        ((Card)e.nextElement()).draw(g, 0, y, this);
      }
      
      y += Y_OFFSET;        
      hid++;
    }        
  }
}
