import java.awt.*;
import java.util.Vector;

/******************************************************************************/
/* class Foundation                                                           */
/*                                                                            */
/* Represents one of the four piles of cards in the top right part of the     */
/* playing area.                                                              */
/******************************************************************************/
public class Foundation extends Pile
{
  /****************************************************************************/
  /* We can take floats if they are aces, or the same suit and one value up.  */
  /****************************************************************************/
  boolean floatIsLegal()
  {
    Floater floater = tableau.getFloater();
    
    if (floater.count() == 0)
    {
      return false;
    }
    
    // Only one card at once...
    if (floater.count() == 1)
    {
      Card c = floater.bottom();
      
      // If the foundation is empty, any ace can be placed on it.
      // Otherwise, we need to match suit and increment value.
      if (isEmpty() && c.getValue() == 1)
      {
        return true;
      }
      else if (!isEmpty() && 
              ((c.getSuit()  == top().getSuit()) &&
               (c.getValue() == top().getValue() + 1)))
      {
        return true;
      }
    }
    
    return false;
  }
 
  /****************************************************************************/
  /* We always float only the top card (if there is one.)                     */
  /****************************************************************************/
  Vector getFloated(int x, int y, Point offset)
  {
    if (!isEmpty())
    {
      return pop(1);
    }
    else
    {
      return null;
    }
  }

  /****************************************************************************/
  /* Stack ops are the same, except that if the stack fills up, we need to    */
  /* check to see if we've filled all four.                                   */
  /****************************************************************************/
  boolean push(Vector v)
  {
    super.push(v);
    
    if (count() == 13)
    {
      tableau.checkForWin();
    }
    
    return true;
  }
  
  /****************************************************************************/
  /* Layout and painting.                                                     */
  /****************************************************************************/
  public Dimension getPreferredSize()
  {
    // Default is the size of one card.
    return new Dimension(Card.WIDTH, Card.HEIGHT);
  }
  
  public void paint(Graphics g)
  {
    if (!isEmpty())
    {
      // Foundations are represented as their top card...
      top().draw(g, 0, 0, this);
    }
    else
    {
      // or an empty square.
      g.drawImage(empty, 0, 0, this);      
    }
  }  
}
  
