import java.applet.Applet;
import java.util.Vector;

/******************************************************************************/
/* class Pack                                                                 */
/*                                                                            */
/* A convenience class for creating, shuffling and distributing a set of      */
/* cards.                                                                     */
/******************************************************************************/
public class Pack
{
  Vector cards = new Vector(52);
  int dealt = 0;                   // Number of cards dealt - only ever goes up.
  
  /****************************************************************************/
  /* Constructor - create the cards.                                          */
  /****************************************************************************/
  Pack(Tableau tableau)
  {
    Applet applet = tableau.getApplet();
    
    Vector suits = Suit.allSuits();

    int cardno;
    int suitno;  
    for (suitno = 0; suitno < 4; suitno++) 
    {
      for (cardno = 1; cardno < 14; cardno++) 
      {
    	  // note that the cards are referenced by integers [1,13]
        Card c = new Card(applet, (Suit)suits.elementAt(suitno), cardno);
  	    cards.addElement(c);
      }
    }
  }  
  
  /****************************************************************************/
  /* shuffle() shuffles the pack.                                             */
  /****************************************************************************/
  public void shuffle() 
  {  
    int size = cards.size(); 
    
    // dispose of the trivial cases first
    if (size <= 1)
    {
      return;
    }

    Vector tempcopy = new Vector(size);
    int i, j;

    // Invariant: i elements have been selected from v
    // and moved to tempcopy, and sizev > 1
    for (i = 0; i < size - 1; i++) 
    {
      // generate a random integer in the range [0, sizev-i)
      // Math.random() handily gives a float in [0, 1.0)
      j = (int)((size - i) * Math.random());
      
      // so add v[j] to tempcopy, then remove it from v
      tempcopy.addElement(cards.elementAt(j));
      cards.removeElementAt(j);
    }
    
    // there is exactly one element left in v at this point
    tempcopy.addElement(cards.elementAt(0));

    cards = tempcopy;
    
    return;
  }
  
  /****************************************************************************/
  /* Return the top i cards from the pack.                                    */
  /****************************************************************************/
  public Vector deal(int count)
  {
    Vector retvect = new Vector(count);
    
    while (dealt < 52 && count > 0)
    {
      retvect.addElement(cards.elementAt(dealt));
      dealt++;
      count--;
    }

    return retvect;
  }
}

