import java.util.Vector;
import java.awt.event.*;
import java.awt.*;

/******************************************************************************/
/* class Talon                                                                */
/*                                                                            */
/* Represents the pile of face-down cards in the top left of the playing      */
/* area.                                                                      */
/******************************************************************************/
public class Talon extends Pile
{
  Waste waste;
  
  /****************************************************************************/
  /* Constructor                                                              */
  /****************************************************************************/
  Talon(Vector cards, Waste waste)
  {
    this.cards = cards;
    this.waste = waste;
    
    /**************************************************************************/
    /* An extra MouseListener to turn over the top three cards.               */
    /**************************************************************************/
    this.addMouseListener(new MouseAdapter() 
    {
      public void mousePressed(MouseEvent e) 
      {        
        twist();
      }
    });  
  }  
  
  /****************************************************************************/
  /* Turn over the next (up to) three cards.                                  */
  /****************************************************************************/
  void twist()
  { 
    /**************************************************************************/
    /* If there are no cards left, claim the Waste back.                      */
    /**************************************************************************/
    if (isEmpty())
    {
      // Copy the waste back onto the talon.
      while (!waste.isEmpty())
      { 
        push(waste.pop(1));
        waste.repaint();
      }
    }
    else
    {
      // Cards get transferred three at a time.
      int i = 0;

      while (i < 3 && !isEmpty())
      {
        waste.push(pop(1));

        i++;
      }    
    }
  }

  /****************************************************************************/
  /* Not allowed to float off the Talon.                                      */
  /****************************************************************************/
  Vector getFloated(int x, int y, Point offset)
  {
    return null;
  }
  
  /****************************************************************************/
  /* Talon doesn't accept floats either.                                      */
  /****************************************************************************/
  boolean floatIsLegal()
  {
    return false;
  }

  /****************************************************************************/
  /* Always the size of a single card.                                        */
  /****************************************************************************/
  public Dimension getPreferredSize()
  {
    // Default is the size of one card.
    return new Dimension(Card.WIDTH, Card.HEIGHT);
  }

  /****************************************************************************/
  /* Paint an empty square or a card back.                                    */
  /****************************************************************************/
  public void paint(Graphics g)
  {
    Image i = isEmpty() ? empty : back;
    
    g.drawImage(i, 0, 0, this);      
  }
}
    
