import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

/******************************************************************************/
/* class Waste                                                                */
/*                                                                            */
/* Represents the pile of face-up cards in the top left of the playing        */
/* area.                                                                      */
/******************************************************************************/
public class Waste extends Pile
{
  /****************************************************************************/
  /* How many cards we are displaying on the top of the waste                 */
  /****************************************************************************/
  int displaycount = 0;
                                          
  /****************************************************************************/
  /* push() and pop() have to update the displaycount, keeping it within      */
  /* 1 <= displacount <= 3.                                                   */
  /****************************************************************************/
  boolean push(Vector v)
  {
    if (displaycount < 3)
    {
      displaycount++;
    }
    
    return super.push(v);
  }
  
  Vector pop(int amount)
  {
    if (displaycount > 1)
    {
      displaycount--;
    }
    
    return super.pop(amount);
  }
  
  /****************************************************************************/
  /* Can't drag onto the Waste.                                               */
  /****************************************************************************/
  boolean floatIsLegal() 
  { 
    return false; 
  }

  /****************************************************************************/
  /* We can drag off the Waste - but only one card, and only if it's clicked  */
  /* on directly.                                                             */
  /****************************************************************************/
  Vector getFloated(int x, int y, Point offset)
  {
    if (!isEmpty() && x > ((displaycount - 1) * X_OFFSET))
    {
      offset.x = offset.x - ((displaycount - 1) * X_OFFSET);
      return pop(1);
    }
    else
    {
      return null;
    }
  }  

  /****************************************************************************/
  /* Always the size of three layered cards.                                  */
  /****************************************************************************/
  public Dimension getPreferredSize()
  {
    return new Dimension(Card.WIDTH + 2 * X_OFFSET, Card.HEIGHT);
  }
  
  public void paint(Graphics g)
  {
    // Painting is a bit complex - we have to start at three cards after
    // every turnover, and reduce to one only as they get removed. But we
    // also have to shift them over if there are e.g. only two left.
    if (cards.isEmpty())
    {
      return;
    }
    
    int x = 0;
    int dc = displaycount;
    
    while (dc > 0)
    { 
      if (count() - dc >= 0)
      {       
        ((Card)cards.elementAt(count() - dc)).draw(g, x, 0, this);
      }
      
      dc--;
      x += X_OFFSET;
    }      
  }
}
 
